/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * boolean.hpp
 *
 *  Created on: 2010/12/20
 *      Author: HARK Development Team
 */

#ifndef BOOLEAN_HPP_
#define BOOLEAN_HPP_

/**
 * Useful boolean type.
 * Replacement of built-in bool type.
 *
 * Features (Some advantages of boolean over bool)
 *   You can use boolean with std::vector!
 *    - When you use std::vector<bool> you will meet some obstacles,
 *      because std::vector<bool> is NOT a standard STL container.
 *      Unlike std::vector<bool>, std::vector<boolean> is a true standard STL container!
 *   "boolean" will always be initialized!
 *    - When you write
 *        bool b;
 *      b will NOT be initialized automatically.
 *      When you write
 *        boolean b;
 *      b will be initialized automatically!
 *   Using boolean has no overhead.
 *    - If you compile your source code with optimize options,
 *      there will be no overhead.
 */
class boolean
{
    bool value;
public:
    /**
     * Creates boolean variable with specified value.
     *
     * This constructor is implicit, so you can write such as:
     *   boolean b = true;
     *
     * @param value initial value
     */
    boolean(bool value = false)
        : value(value)
    { }

    /**
     * Converts boolean to built-in bool.
     *
     * This conversion operator makes it possible to write
     *   boolean b1;
     *   bool b2 = b1;
     *
     * @return converted value
     */
    operator bool() const
    {
        return this->value;
    }
};

#endif /* BOOLEAN_HPP_ */
